/******************************************************************************
 *
 * Copyright (c) 1998,99 by Mindbright Technology AB, Stockholm, Sweden.
 *                 www.mindbright.se, info@mindbright.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *****************************************************************************
 * $Author: mats $
 * $Date: 1999/01/31 19:00:36 $
 * $Name: rel0-98-1 $
 *****************************************************************************/
package mindbright.applet;

import java.applet.Applet;

import java.awt.*;
import java.awt.event.*;
import java.util.NoSuchElementException;

import mindbright.ssh.*;
import mindbright.security.*;
import mindbright.terminal.*;

/* !!! TESTING !!!
import netscape.security.Target;
import netscape.security.Principal;
import netscape.security.PrivilegeManager;
   !!! TESTING !!! */

public class MindTerm extends Applet {

  Frame       frame;
  TerminalWin term;
  Thread      clientThread;
  int         rows;
  int         cols;

  SSHInteractiveClient client;
  SSHStdIO             console;

  static public class WaiterThread implements Runnable {
    Thread   master;
    MindTerm appl;
    public WaiterThread(Thread master, MindTerm appl) {
      this.master = master;
      this.appl   = appl;
    }
    public void run() {
      try {
	master.join();
      } catch (Exception e) {
      }
      appl.windowClosing(null);
    }
  }
  
  public void init() {
    boolean verbose        = false;
    boolean separateFrame  = true;
    boolean usePropFiles   = false; // Not used right now...
    boolean useDefPropOnly = false; // Not used right now...
    int     i;
    String  param;
    
    try {
      separateFrame = (new Boolean(getParameter("sepframe"))).booleanValue();
    } catch (Exception e) {
      separateFrame = true;
    }
    try {
      rows = Integer.parseInt(getParameter("rows"));
    } catch (Exception e) {
      rows = 24;
    }
    try {
      cols = Integer.parseInt(getParameter("cols"));
    } catch (Exception e) {
      cols = 80;
    }
    try {
      verbose = (new Boolean(getParameter("verbose"))).booleanValue();
    } catch (Exception e) {
      verbose = false;
    }

    // !!! TODO !!!
    //
    client  = new SSHInteractiveClient("http:www.example.com/mysettings/", null);
    console = (SSHStdIO)client.getConsole();
    client.setUsePropertyFiles(usePropFiles);
    client.setUseOnlyDefaultProperties(useDefPropOnly);

    initAppletGUI(separateFrame);

    // Default server to connect to is of course the origin of the applet
    //
    client.setProperty("server", getCodeBase().getHost());

    // Get applet-parameters corresponding to properties
    //
    for(i = 0; i < SSHInteractiveClient.defaultPropDesc.length; i++) {
      param = getParameter(client.defaultPropDesc[i][client.PROP_NAME]);
      try {
      if(param != null)
	client.setProperty(client.defaultPropDesc[i][client.PROP_NAME], param);
      } catch (NoSuchElementException e1) {
	console.println("No such property: " + param);
      } catch (IllegalArgumentException e2) {
	console.println(e2.getMessage());
      }
    }

    // !!! To prevent problems if someone blindly copy params... (remove when URL-loading done)
    //
    client.setProperty("idhost", "false");

    for(i = 0; i < term.defaultPropDesc.length; i++) {
      param = getParameter(term.defaultPropDesc[i][term.PROP_NAME]);
      try {
      if(param != null)
	term.setProperty(term.defaultPropDesc[i][term.PROP_NAME], param);
      } catch (NoSuchElementException e1) {
	console.println("No such property: " + param);
      } catch (IllegalArgumentException e2) {
	console.println(e2.getMessage());
      }
    }

    console.setTerminal(term);
    if(separateFrame)
      console.setOwnerContainer(frame);
    else
      console.setOwnerContainer(this);
    console.setOwnerName(SSH.VER_MINDTERM);
    console.updateTitle();

    client.printCopyright();

    /* !!! TESTING !!!
    Principal          sysPrincipal = null;
    PrivilegeManager   privMgr = null;
    Target             propReadTarget = null;

    System.out.println("Initializing PrivilegeManager and System Principal.");
    privMgr =      PrivilegeManager.getPrivilegeManager();
    sysPrincipal =   PrivilegeManager.getSystemPrincipal();
    System.out.println("Trying to acquire permission to read system properties...");
    try {
      propReadTarget = Target.findTarget("UniversalConnect", sysPrincipal);
      privMgr.enablePrivilege(propReadTarget);
      System.out.println("** Success?");
    } catch( Exception e ) {
      System.out.println( "Couldn't enable Netscape Navigator security options: " + e.toString() );
      // Something went wrong, so forget about the whole affair
    }
    !!! TESTING !!! */

    // Adhoc... Let the AWT-stuff settle before we try to print to the term...
    // (TODO: get rid of this, should wait for something more specific I guess :-)
    //
    try {
      Thread.sleep(1500);
    } catch(Exception e) {
    }

    clientThread = new Thread(client);
    clientThread.start();

    // Launch a thread waiting for the client to die to be able to remove the frame
    // (if we are in a separate one)
    (new Thread(new WaiterThread(clientThread, this))).start();
  }

  public void initAppletGUI(boolean separateFrame) {
    if(separateFrame) {
      frame = new Frame();
      term  = new TerminalWin(frame, cols, rows, new TerminalXTerm());
      frame.setSize(term.getPreferredSize());
      frame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e)  { MindTerm.this.windowClosing(e); }
      });
      frame.add(term.getPanelWithScrollbar());
      frame.pack();
      frame.show();
    } else {
      frame = (Frame)getParent();
      term  = new TerminalWin(frame, cols, rows, new TerminalXTerm());
      this.add(term.getPanelWithScrollbar());
    }

    term.addMenuItem(null);
    SSHMenuHandler menus = new SSHMenuHandler(client, frame, term);
    term.addMenuItem(menus.getMindTermMenu());
    term.addMenuItem(menus.getSettingsMenu());
    term.addMenuItem(menus.getTunnelsMenu());
  }

  public void windowClosing(WindowEvent e) {
    System.out.println("Thank you for using MindTerm...");
    if(frame != null) {
      frame.dispose();
    } else {
      int i;
      try {
	term.cursorSetPos(term.rows() - 1, 0, false);
	for(i = 0; i < term.rows(); i++) {
	  term.write("\n\r");
	  Thread.sleep(50);
	}
	term.cursorSetPos(0, 0, false);
	for(i = 0; i < term.rows() - 1; i++) {
	  term.write(".\n\r");
	  Thread.sleep(50);
	}
	term.write("Thank you for using MindTerm...");
	for(i = 0; i < term.rows() - 1; i++) {
	  term.write("\n\r");
	  Thread.sleep(50);
	}
	term.cursorSetPos(2, 0, false);
	term.setAttribute(TerminalWin.ATTR_BOLD, true);
	term.write("Visit <http://www.mindbright.se/mindterm> for more information.");
	term.cursorSetPos(term.rows() - 1, term.cols() - 1, false);
      } catch (Exception ee) {
	// !!!
      }
    }
    if(clientThread.isAlive())
      clientThread.stop();
  }

}
