/******************************************************************************
 *
 * Copyright (c) 1998,99 by Mindbright Technology AB, Stockholm, Sweden.
 *                 www.mindbright.se, info@mindbright.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *****************************************************************************
 * $Author: mats $
 * $Date: 1999/02/01 07:27:00 $
 * $Name: rel0-98-1 $
 *****************************************************************************/
package mindbright.ssh;

import java.awt.*;
import java.awt.event.*;

import mindbright.terminal.TerminalWin;

public class SSHMenuHandler extends Object {

  SSHInteractiveClient client;
  Frame                parent;
  TerminalWin          term;

  static SSHMenuHandler menus;

  public SSHMenuHandler(SSHInteractiveClient client, Frame parent, TerminalWin term) {
    this.client = client;
    this.parent = parent;
    this.term   = term;

    menus = this;
  }

  MenuItem disconnect;
  MenuItem loadDef;
  MenuItem saveDef;
  MenuItem commandSh;
  public MenuItem getMindTermMenu() {
    Menu m = new Menu("MindTerm Main");
    m.add(disconnect = new MenuItem("Disconnect from sshd"));
    m.add(loadDef    = new MenuItem("Load default properties"));
    m.add(saveDef    = new MenuItem("Save default properties"));
    m.add(commandSh  = new MenuItem("Local command-shell"));

    disconnect.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	client.forcedDisconnect();
      }
    });

    loadDef.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	client.loadDefaultProperties();
      }
    });

    saveDef.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	client.saveDefaultProperties();
      }
    });

    commandSh.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	client.console.println("");
	client.console.println("** hit a key to enter local command-shell **");
	((SSHStdIO)(client.console)).wantCommandShell();
      }
    });

    updateMindTermMenu();
    return m;
  }

  public void updateMindTermMenu() {
    if(client.isConnected) {
      disconnect.setEnabled(true);
      loadDef.setEnabled(false);
      if(client.useDefPropOnly || !client.usePropFiles) {
	saveDef.setEnabled(false);
      } else {
	saveDef.setEnabled(true);
      }
      commandSh.setEnabled(true);
    } else {
      disconnect.setEnabled(false);
      if(client.useDefPropOnly || !client.usePropFiles) {
	loadDef.setEnabled(false);
	saveDef.setEnabled(false);
      } else {
	loadDef.setEnabled(true);
	saveDef.setEnabled(true);
      }
      commandSh.setEnabled(false);
    }
  }

  public static void updateMenus() {
    menus.updateMindTermMenu();
  }

  public MenuItem getSettingsMenu() {
    MenuItem mi = new MenuItem("SSH Settings");
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sshSettingsMenu();
      }
    });
    return mi;
  }

  public MenuItem getTunnelsMenu() {
    MenuItem mi = new MenuItem("SSH Tunnels");
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	sshTunnelsMenu();
      }
    });
    return mi;
  }

  Dialog     settingsDialog = null;
  Choice     choiceCipher, choiceAuthTyp, choiceX11, choicePrvPrt, choiceRemFwd, choiceIdHost;
  TextField  textSrv, textPort, textUser, textId, textDisp, textMtu;
  Label      lblAlert, lblSrv;
  String[]   cipher, authtyp, bool;
  Panel      srvPanel;
  CardLayout srvCard;
  public final void sshSettingsMenu() {
    int   i;

    //if(settingsDialog == null) {
    // !!! BUG, when the Dialog is shown repeatedly the bottom panel ("South") slides uppwards... !!!
    if(true) {
      cipher  = SSH.getCiphers();
      authtyp = SSH.getAuthTypes();
      bool    = new String[2];
      bool[0] = "true";
      bool[1] = "false";

      settingsDialog = new Dialog(parent, "SSH Settings", true) {
	public Insets getInsets() { Insets ins = super.getInsets();
				    ins.left = 5; ins.right = 5; ins.bottom += 5; ins.top += 5; return ins; }
      };

      settingsDialog.setLayout(new BorderLayout(5, 5));

      Panel p = new Panel(new GridLayout(12, 2, 2, 2));
      
      p.add(new Label("Server:"));

      srvPanel = new Panel(srvCard = new CardLayout());
      srvPanel.add("text", textSrv = new TextField("", 12));
      srvPanel.add("label", lblSrv = new Label(""));
      p.add(srvPanel);

      p.add(new Label("Port:"));
      p.add(textPort = new TextField("", 12));

      p.add(new Label("Username:"));
      p.add(textUser = new TextField("", 12));
    
      p.add(new Label("Cipher:"));
      choiceCipher = new Choice();
      for(i = 0; i < cipher.length; i++) {
	choiceCipher.add(cipher[i]);
      }
      p.add(choiceCipher);

      p.add(new Label("Type of auth.:"));
      choiceAuthTyp = new Choice();
      for(i = 0; i < authtyp.length; i++) {
	choiceAuthTyp.add(authtyp[i]);
      }
      p.add(choiceAuthTyp);

      p.add(new Label("Identity file:"));
      p.add(textId = new TextField("", 12));

      p.add(new Label("Display (x11-fwd):"));
      p.add(textDisp = new TextField("", 12));

      p.add(new Label("Max. pkt size:"));
      p.add(textMtu = new TextField("", 12));

      p.add(new Label("Allow X11 forward:"));
      choiceX11 = new Choice();
      choiceX11.add(bool[0]);
      choiceX11.add(bool[1]);
      p.add(choiceX11);

      p.add(new Label("Use priv. src. port:"));
      choicePrvPrt = new Choice();
      choicePrvPrt.add(bool[0]);
      choicePrvPrt.add(bool[1]);
      p.add(choicePrvPrt);

      p.add(new Label("Allow remote conn.:"));
      choiceRemFwd = new Choice();
      choiceRemFwd.add(bool[0]);
      choiceRemFwd.add(bool[1]);
      p.add(choiceRemFwd);

      p.add(new Label("Check ID of server:"));
      choiceIdHost = new Choice();
      choiceIdHost.add(bool[0]);
      choiceIdHost.add(bool[1]);
      p.add(choiceIdHost);

      settingsDialog.add(p, BorderLayout.CENTER);

      p = new Panel(new GridLayout(2, 1, 3, 3));
      p.add(lblAlert = new Label("", Label.CENTER));

      Panel bp = new Panel(new FlowLayout());

      Button b;
      bp.add(b = new Button("Cancel"));
      b.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  settingsDialog.setVisible(false);
	}
      });
      bp.add(b = new Button("OK"));
      b.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  try {
	    if(!client.isConnected)
	      client.setProperty("server", textSrv.getText());
	    client.setProperty("port", textPort.getText());
	    client.setProperty("usrname", textUser.getText());
	    client.setProperty("cipher", cipher[choiceCipher.getSelectedIndex()]);
	    client.setProperty("authtyp", authtyp[choiceAuthTyp.getSelectedIndex()]);
	    client.setProperty("idfile", textId.getText());
	    client.setProperty("display", textDisp.getText());
	    client.setProperty("mtu", textMtu.getText());
	    client.setProperty("x11fwd", bool[choiceX11.getSelectedIndex()]);
	    client.setProperty("prvport", bool[choicePrvPrt.getSelectedIndex()]);
	    client.setProperty("remfwd", bool[choiceRemFwd.getSelectedIndex()]);
	    client.setProperty("idhost", bool[choiceIdHost.getSelectedIndex()]);
	    settingsDialog.setVisible(false);
	  } catch (Exception ee) {
	    lblAlert.setText(ee.getMessage());
	  }
	}
      });
      p.add(bp);

      settingsDialog.add(p, BorderLayout.SOUTH);

      settingsDialog.setResizable(false);
      settingsDialog.pack();
    }

    textSrv.setText(client.getProperty("server"));
    lblSrv.setText(client.getProperty("server"));
    if(!client.isConnected)
      srvCard.first(srvPanel);
    else
      srvCard.last(srvPanel);
      

    textPort.setText(client.getProperty("port"));

    textUser.setText(client.getProperty("usrname"));

    choiceCipher.select(client.getProperty("cipher"));
    choiceAuthTyp.select(client.getProperty("authtyp"));

    textId.setText(client.getProperty("idfile"));
    textDisp.setText(client.getProperty("display"));
    textMtu.setText(client.getProperty("mtu"));

    choiceX11.select(client.getProperty("x11fwd"));
    choicePrvPrt.select(client.getProperty("prvport"));
    choiceRemFwd.select(client.getProperty("remfwd"));
    choiceIdHost.select(client.getProperty("idhost"));

    lblAlert.setText("");

    placeDialog(settingsDialog);
    settingsDialog.setVisible(true);
  }

  TunnelEditor localEdit    = null;
  TunnelEditor remoteEdit   = null;
  Dialog       tunnelDialog = null;

  public final void sshTunnelsMenu() {
    if(tunnelDialog == null) {
      tunnelDialog = new Dialog(parent, "SSH Tunnels", true) {
	public Insets getInsets() { Insets ins = super.getInsets();
				    ins.left = 5; ins.right = 5; ins.bottom += 5; ins.top += 5; return ins; }
      };
      tunnelDialog.setLayout(new BorderLayout(5, 5));

      localEdit  = new TunnelEditor("Local: ([/plug/]<loc-prt>:<rem-host>:<rem-prt>)", new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  try {
	    client.addLocalPortForward(localEdit.getText(), false);
	    localEdit.addToList(localEdit.getText());
	  } catch (Exception e1) {
	    localEdit.selectText();
	  }
	}
      });

      tunnelDialog.add(localEdit, BorderLayout.NORTH);

      remoteEdit = new TunnelEditor("Remote: ([/plug/]<rem-prt>:<loc-host>:<loc-prt>)", new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  try {
	    client.addRemotePortForward(remoteEdit.getText(), false);
	    remoteEdit.addToList(remoteEdit.getText());
	  } catch (Exception e2) {
	    remoteEdit.selectText();
	  }
	}
      });

      tunnelDialog.add(remoteEdit, BorderLayout.CENTER);

      Button b;
      Panel p = new Panel(new FlowLayout());
      p.add(b = new Button("Cancel"));
      b.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  tunnelDialog.setVisible(false);
	}
      });
      p.add(b = new Button("OK"));
      b.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int ic;
	  for(ic = 0; ic < localEdit.getItemCount(); ic++)
	    client.setProperty("local" + ic, localEdit.getItem(ic));
	  for(ic = 0; ic < remoteEdit.getItemCount(); ic++)
	    client.setProperty("remote" + ic, remoteEdit.getItem(ic));
	  tunnelDialog.setVisible(false);
	}
      });
      tunnelDialog.add(p, BorderLayout.SOUTH);
      tunnelDialog.setResizable(false);
      tunnelDialog.pack();
    } else {
      localEdit.removeAll();
      remoteEdit.removeAll();
    }
    int i;
    for(i = 0; i < client.getNumOfLocalForwards(); i++)
      localEdit.addToList(client.getProperty("local" + i));
    for(i = 0; i < client.getNumOfRemoteForwards(); i++)
      remoteEdit.addToList(client.getProperty("remote" + i));

    placeDialog(tunnelDialog);
    tunnelDialog.setVisible(true);
  }

  void placeDialog(Dialog diag) {
    Dimension sDim = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension mDim = diag.getSize();
    int x, y;
    x = ((sDim.width / 2) - (mDim.width / 2));
    y = ((sDim.height / 2) - (mDim.height / 2));
    diag.setLocation(x, y);
  }

  protected class TunnelEditor extends Panel {
    List      list;
    TextField text;
    boolean   isLocal;

    public TunnelEditor(String head, ActionListener al) {
      super(new BorderLayout(5, 5));

      Panel  pi;
      Button b;

      add(new Label(head), BorderLayout.NORTH);
      add(list = new List(5, false), BorderLayout.CENTER);
      pi = new Panel(new FlowLayout());
      pi.add(text = new TextField("", 18));
      pi.add(b = new Button("Add"));
      b.addActionListener(al);
      pi.add(b = new Button("Delete"));
      b.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int i = list.getSelectedIndex();
	  if(i != -1) {
	    list.remove(i);
	  }
	}
      });
      add(pi, BorderLayout.SOUTH);
      list.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  text.setText(list.getSelectedItem());
	}
      });
    }

    public int getItemCount() {
      return list.getItemCount();
    }

    public String getItem(int i) {
      return list.getItem(i);
    }

    public void addToList(String item) {
      list.add(item);
    }

    public void selectText() {
      text.selectAll();
    }

    public String getText() {
      return text.getText();
    }

    public void removeAll() {
      list.removeAll();
    }
  }



}
